<?php
  function oms_buyer_type($mortgageType, $purchaserType) {
    if ($mortgageType === 2){
      return 'Remortgage';
    }
    return $purchaserType === 1 ? 'FirstTimeBuyer' : 'Purchase';
  }

  function oms_employment_status($employmentStatus){
    switch ($employmentStatus) {
      case 1: // Employed
        return 'Employed';
      case 2: // SelfEmployed
      case 3: // DirectorOrShareholder
        return 'SelfEmployed';
      default: // Homemaker, Retired, Student, Unemployed
        return 'Unemployed';
    }
  }

  function oms_get_applicants($applicants, $callBack) {
    $mappedApplicants = [];
    foreach($applicants as $applicant) {
      array_push($mappedApplicants, [
        'annualIncome' => $applicant['primaryIncome']['grossAnnualIncome'],
        'email' => $callBack['emailAddress'],
        'employmentStatus' => oms_employment_status($applicant['primaryIncome']['employmentStatus']),
        'selfEmploymentType' => $applicant['primaryIncome']['employmentStatus'] === 2 ? 'SoleTrader' : '',
        'firstName' => $callBack['firstName'],
        'lastName' => $callBack['lastName'],
        'telephone' => $callBack['phoneNumber']
      ]);
    }

    return $mappedApplicants;
  }

  function base64url_encode($str) {
    return rtrim(strtr(base64_encode($str), '+/', '-_'), '=');
  }

  function generate_jwt($headers, $payload, $secret = 'secret') {
    $headers_encoded = base64url_encode(json_encode($headers));

    $payload_encoded = base64url_encode(json_encode($payload));

    $signature = hash_hmac('SHA256', "$headers_encoded.$payload_encoded", base64_decode($secret), true);
    $signature_encoded = base64url_encode($signature);

    $jwt = "$headers_encoded.$payload_encoded.$signature_encoded";

    return $jwt;
  }

  function oms_generate_lead($mappedData) {
    $time = time();
    $headers = array('alg' => 'HS256', 'typ' => 'JWT');
    $payload = array(
      'nbf' => $time,
      'iat' => $time,
      'aud' => $GLOBALS['mcSettings']['MC_OMS_aud'],
      'exp' => ($time + 60000)
    );
    $jwt = generate_jwt($headers, $payload, $GLOBALS['mcSettings']['MC_OMS_base_64']);
    $body = $mappedData;

    $options = array(
      'headers' => array(
        'Content-Type' => 'application/json; charset=utf-8',
        'APIUser' => $GLOBALS['mcSettings']['MC_OMS_API_user'],
        'Authorization' => "Bearer $jwt"
      ),
      'body' => json_encode($body),
      'sslverify' => $GLOBALS['mcSettings']["MC_environment"] === 'dev' ? FALSE : TRUE
    );

    $response = wp_remote_post("https://demo.omsystem.co.uk/api/leadgen/case/v1/enquiry", $options);
    return $response;
  }

  function toDateTime($dob) {
    if (!$dob) {
      return '';
    };
    $date = date_create($dob['year'].'-'.$dob['month'].'-'.$dob['day']);
    $dateFormat = date_format($date, 'Y-m-d');
    return $dateFormat;
  }

  function map_to_OMS($request) {
    $propertyAndLoan = $request['inputs']['propertyAndLoan'];
    $applicants = $request['inputs']['applicants'];
    $incomeAndExpenditure = $request['inputs']['incomeAndExpenditure'];
    $callBack = $request['callback'];
    $mappedData = [
        'BrokerEmail' => $GLOBALS['mcSettings']['MC_OMS_broker_email'],
        'WorkflowName' => $GLOBALS['mcSettings']['MC_OMS_workflow_name'],
        'Enquiries' => array([
          "EnquiryType"=> 0,
          "IntroductionType"=> 1,
          "MortgageType"=> $propertyAndLoan['mortgageType'],
          "PurchasePrice"=> $propertyAndLoan['propertyValue'],
          "LoanAmount"=> $propertyAndLoan['loanAmount'],
          "Term"=> $propertyAndLoan['mortgageTerm']['years'],
          "OutstandingBalance" => $propertyAndLoan['mortgageType'] !== 'Purchase'
            ? $propertyAndLoan['amountTransferredFromOtherLender']
            : null,
          "Summary" => "Callback request via Legal and General SmartrFit Affordability Calculator. Date and Time: $callBack[callPreference] $callBack[callBackDate] $callBack[callBackTime] " ,
          "ExistingPropertiesOwned"=> $incomeAndExpenditure['otherMortgages'] > 4
            ? 4
            : $incomeAndExpenditure['otherMortgages'],
          "Applicant1Title"=> 'Mx',
          "Applicant1FirstName"=> $callBack['firstName'],
          "Applicant1LastName"=> $callBack['lastName'],
          "Applicant1EmailAddress"=> $callBack['emailAddress'],
          "Applicant1PhoneNo"=> $callBack['phoneNumber'],
          "Applicant1Dob"=> toDateTime($applicants['applicantAges'][0]['dateOfBirth']),
          "Applicant1EmployedIncome"=> $incomeAndExpenditure['applicants'][0]['primaryIncome']['grossAnnualIncome'],
          "Applicant1SelfEmployedIncome"=> $incomeAndExpenditure['applicants'][0]['primaryIncome']['netProfitForLatestPeriod'],
          "Applicant2Dob"=> toDateTime($applicants['applicantAges'][1]['dateOfBirth']),
          "Applicant2EmployedIncome"=> $incomeAndExpenditure['applicants'][1]['primaryIncome']['grossAnnualIncome'],
          "Applicant2SelfEmployedIncome"=> $incomeAndExpenditure['applicants'][1]['primaryIncome']['netProfitForLatestPeriod'],
      ]),
    ];

    $response = oms_generate_lead($mappedData);

    return $response;
  }
?>
