<?php
  function s365_buyer_type($mortgageType, $purchaserType) {
    if ($mortgageType === 2){
      return 'Remortgage';
    }
    return $purchaserType === 1 ? 'FirstTimeBuyer' : 'Purchase';
  }

  function s365_employment_status($employmentStatus){
    switch ($employmentStatus) {
      case 1: // Employed
        return 'Employed';
      case 2: // SelfEmployed
      case 3: // DirectorOrShareholder
        return 'SelfEmployed';
      default: // Homemaker, Retired, Student, Unemployed
        return 'Unemployed';
    }
  }

  function s365_get_applicants($applicants, $callBack) {
    $mappedApplicants = [];
    foreach($applicants as $applicant) {
      array_push($mappedApplicants, [
        'annualIncome' => $applicant['primaryIncome']['grossAnnualIncome'],
        'email' => $callBack['emailAddress'],
        'employmentStatus' => s365_employment_status($applicant['primaryIncome']['employmentStatus']),
        'selfEmploymentType' => $applicant['primaryIncome']['employmentStatus'] === 2 ? 'SoleTrader' : '',
        'firstName' => $callBack['firstName'],
        'lastName' => $callBack['lastName'],
        'telephone' => $callBack['phoneNumber']
      ]);
    }

    return $mappedApplicants;
  }

  function s365_generate_lead($mappedData) {
    $body = $mappedData;

    $options = array(
      'headers' => array(
        'X-API-KEY' => $GLOBALS['mcSettings']['MC_S365_API_key'],
        'Content-Type' => 'application/json; charset=utf-8'
      ),
      'body' => json_encode($body),
      'sslverify' => $GLOBALS['mcSettings']["MC_environment"] === 'dev' ? FALSE : TRUE
    );

    $response = wp_remote_post("https://api-sandbox.smartr365.com/api/v1/mortgage/lead/create", $options);

    return $response;
  }

  function map_to_S365($request) {
    $propertyAndLoan = $request['inputs']['propertyAndLoan'];
    $applicants = $request['inputs']['applicants'];
    $incomeAndExpenditure = $request['inputs']['incomeAndExpenditure'];
    $callBack = $request['callback'];
    $mappedData = [
        'buyerType' => s365_buyer_type($propertyAndLoan['mortgageType'], $propertyAndLoan['purchaserType']),
        'currentPropertyValuation' => $propertyAndLoan['propertyValue'],
        'depositAmount' => $propertyAndLoan['mortgageType'] !== 2
          ? $propertyAndLoan['propertyValue'] - $propertyAndLoan['loanAmount']
          : null,
        'followUpDate' => $callBack['callPreference'] === 'preferred'
          ? $callBack['callBackDate'] . ' ' . $callBack['callBackTime']
          : null,
        'introducerId' => null,
        'introducerBranchId' => null,
        'introducerNegotiatorId' => null,
        'jointMortgage' => $applicants['numberOfApplicants'] > 1,
        'leadSource' => 'SmartrFit',
        'mortgageProductType' => 'Residential',
        'mortgageSubType' => 'BridgingFinance',
        'note' => null,
        'offerAcceptedOnProperty' => null,
        'offerQualified' => null,
        'organisationId' => $GLOBALS['mcSettings']['MC_S365_organisation_id'],
        'outstandingMortgageAmount' => $propertyAndLoan['mortgageType'] === 2
          ? $propertyAndLoan['amountTransferredFromOtherLender']
          : null,
        'purchasePrice' => $propertyAndLoan['mortgageType'] !== 2
          ? $propertyAndLoan['propertyValue']
          : null,
        'raiseFundsSubType' => null,
        'raiseFundsAmount' => $propertyAndLoan['mortgageType'] === 2
          ? 0
          : null,
        'raisingAdditionalFunds' => $propertyAndLoan['mortgageType'] === 2
          ? true
          : null,
        'applicants' => s365_get_applicants($incomeAndExpenditure['applicants'], $callBack),
        'adviserId' => 0
    ];

    $response = s365_generate_lead($mappedData);

    return $response;
  }
?>
