<?php
  function get_auth_key($mcSettings, $authEndpoint, $request = null){
    $body = [
      'grant_type' => 'client_credentials',
      'client_id' => $request ? $request['clientId'] : $mcSettings['MC_client_id'],
      'client_secret' => $request ? $request['clientSecret'] : $mcSettings['MC_client_secret'],
      'scope' => 'criteria'
    ];

    $options = array(
      'body' => $body,
      'sslverify' => $mcSettings["MC_environment"] === 'dev' ? FALSE : TRUE
    );

    $response = wp_remote_post($authEndpoint, $options);

    if (is_wp_error($response)) {
      $error_message = $response->get_error_message();
      return [
        'error' => $error_message
      ];
    }

    $body = get_object_vars(json_decode($response['body']));
    if (array_key_exists('error', $body)) {
      $error = $body['error'];
      return [
        'error' => $error
      ];
    }

    $key = $body['access_token'];
    if (!($key && $mcSettings['MC_redirect_url'] && $mcSettings['MC_results_email'])){
      return [
        'error' => 'Could not find settings'
      ];
    }

    return [
      'key' => $key,
      'error' => ''
    ];
  }

  function get_encrypted_settings($mcSettings, $key, $apiEndpoint) {
    // Get encrypted settings
    $body = [
      'redirectUrl' => $mcSettings['MC_redirect_url'],
      'resultsEmail' => $mcSettings['MC_results_email'],
      'clientId' => $mcSettings['MC_client_id'],
      'newBuild' => array_key_exists('MC_new_build', $mcSettings) ? boolval($mcSettings['MC_new_build']) : false,
    ];

    $options = array(
      'headers' => array(
        'Authorization' => 'Bearer ' . $key,
        'Content-Type' => 'application/json; charset=utf-8'
      ),
      'body' => json_encode($body),
      'sslverify' => $mcSettings["MC_environment"] === 'dev' ? FALSE : TRUE
    );

    $response = wp_remote_post("$apiEndpoint/SmartrPlus/EncryptSettings", $options);

    if (is_wp_error($response)) {
      $error_message = $response->get_error_message();
      return [
        'error' => $error_message
      ];
    }
    $body = json_decode($response['body']);
    if ($body->error) {
      $error = $body->error;
      return [
        'error' => $error
      ];
    }

    $encryptedSettings = urlencode($body->result->encryptedSettings);

    return [
      'encryptedSettings' => $encryptedSettings,
      'error' => ''
    ];
  }

  function get_endpoints($mcSettings, $environment = null){
    $authEndpoint = '';
    $apiEndpoint = '';
    $uiEndpoint = '';
    switch ($environment ? $environment : $mcSettings["MC_environment"]) {
      case 'dev':
        $apiEndpoint = "https://localhost:5050/api";
        $authEndpoint = "https://localhost:5070/connect/token";
        $uiEndpoint = "http://localhost:4200";
        break;
      case 'staging':
        $apiEndpoint = "https://mc-smartrcriteria-api-staging.azurewebsites.net/api";
        $authEndpoint = "https://mc-smartrcriteria-sso-staging.azurewebsites.net/connect/token";
        $uiEndpoint = "https://smartr-plus-master.theidoluat.com";
        break;
      case 'live':
        $apiEndpoint = "https://api.mc.legalandgeneral.com/api";
        $authEndpoint = "https://sso.mc.legalandgeneral.com/connect/token";
        $uiEndpoint = "https://smartr-plus-live.theidolprod.com";
        break;
      default: //uat
        $apiEndpoint = "https://mc-smartrcriteria-api-uat.azurewebsites.net/api";
        $authEndpoint = "https://mc-smartrcriteria-sso-uat.azurewebsites.net/connect/token";
        $uiEndpoint = "https://smartr-plus-dev.theidoluat.com";
        break;
    }
    return [
      'apiEndpoint' => $apiEndpoint,
      'authEndpoint' => $authEndpoint,
      'uiEndpoint' => $uiEndpoint,
    ];
  }

  function iframe_script() {
    // render the iframe script
    $mcSettings = get_option('MC_settings');
    $endpoints = get_endpoints($mcSettings);
    $authEndpoint = $endpoints['authEndpoint'];
    $apiEndpoint = $endpoints['apiEndpoint'];
    $uiEndpoint = $endpoints['uiEndpoint'];

    // Get Auth Key
    $response = get_auth_key($mcSettings, $authEndpoint);
    if ($response['error'] !== ''){
      $error = $response['error'];
      return "Something went wrong: $error";
    }
    $key = $response['key'];

    // Get encrypted settings
    $response = get_encrypted_settings($mcSettings, $key, $apiEndpoint);
    if ($response['error'] !== ''){
      $error = $response['error'];
      return "Something went wrong: $error";
    }
    $encryptedSettings = $response['encryptedSettings'];

    $mcSettings['MC_color'] = str_replace('#', '', $mcSettings['MC_color']);

    // Generate HTML body
    ob_start();
    include(plugin_dir_path( __FILE__ ) . 'html/iframe.php');
    $html = ob_get_clean();
    ob_end_clean();
    return $html;
  }
?>
