<?php
  function isJson($string) {
    json_decode($string);
    return json_last_error() === JSON_ERROR_NONE;
  }

  function map_to_crm(WP_REST_Request $request) {
    switch ($GLOBALS['mcSettings']['MC_CRM_system']) {
      case 'S365':
        include(plugin_dir_path( __FILE__ ) . 'crm-maps/S365.php');
        $postResponse = map_to_S365($request);
        break;
      case 'OMS':
        include(plugin_dir_path( __FILE__ ) . 'crm-maps/OMS.php');
        $postResponse = map_to_OMS($request);
        break;

      default:
        $postResponse = json_decode ('{}');
        break;
    }

    if (is_wp_error($postResponse)) {
      $error_message = $postResponse->get_error_message();
      return [
        'error' => $error_message
      ];
    }

    $body = isJson($postResponse['body'])
      ? json_decode($postResponse['body'])
      : $postResponse['body'];
    if ($body->error) {
      $error = $body->error;
      return [
        'error' => $error
      ];
    }
    $response = new WP_REST_Response($body, $postResponse['response']['code']);
    // Set headers.
    $response->set_headers([ 'Cache-Control' => 'no-cache, no-store, private' ]);

    return $response;
  }

  function add_custom_crm_post() {
    // route url: domain.com/wp-json/$namespace/$route
    $namespace = 'mc_api/v1';
    $route     = '/map_to_crm/';

    register_rest_route($namespace, $route, [
      'methods'  => 'POST',
      'callback' => 'map_to_crm',
    ]);
  }
?>
