const emailRegex = /^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
window.onload = function() {
  var form = document.getElementById('settingsForm');
  form.onsubmit = submitted.bind(form);
  document.getElementById('color').addEventListener('change', () => {
    updateRatio();
  });

  document.getElementById('resultsEmail').addEventListener('focusout', () => {
    const emailEl = document.getElementById('resultsEmail');
    if (!emailRegex.test(String(emailEl.value).toLowerCase())) {
      document.getElementById('resultsEmailError').hidden = false;
      document.getElementById('submit').disabled = true;
      document.getElementById('resultsEmail').setCustomValidity('error');
    } else {
      document.getElementById('resultsEmailError').hidden = true;
      document.getElementById('submit').disabled = false;
      document.getElementById('resultsEmail').setCustomValidity('');
    }
  });

  updateRatio();
  crmChanged();
  checkEnableTestButton();
  checkEnableCrmTestButton();
}

function updateRatio() {
  const ratio = document.getElementById('ratio');
  const contrast = getContrast(document.getElementById('color').value);
  ratio.innerText = contrast;
  if (contrast < 4.5) {
    document.getElementById('submit').disabled = true;
    document.getElementById('colorError').classList.add('show-error');
    ratio.classList.add('error');
    ratio.classList.remove('pass');
  } else {
    document.getElementById('submit').disabled = false;
    document.getElementById('colorError').classList.remove('show-error');
    ratio.classList.add('pass');
    ratio.classList.remove('error');
  }
}

function getLuminance(r, g, b) {
  var a = [r, g, b].map(function(v) {
    v /= 255;
    return v <= 0.03928 ?
      v / 12.92 :
      Math.pow((v + 0.055) / 1.055, 2.4);
  });
  return a[0] * 0.2126 + a[1] * 0.7152 + a[2] * 0.0722;
}

function hexToRGB(h) {
  let r = 0,
    g = 0,
    b = 0;
  // 3 digits
  if (h.length == 4) {
    r = '0x' + h[1] + h[1];
    g = '0x' + h[2] + h[2];
    b = '0x' + h[3] + h[3];

    // 6 digits
  } else if (h.length == 7) {
    r = '0x' + h[1] + h[2];
    g = '0x' + h[3] + h[4];
    b = '0x' + h[5] + h[6];
  }
  return [r, g, b];
}

function getContrast(color) {
  const rgb1 = hexToRGB(color);
  const rgb2 = hexToRGB('#fff');
  var lum1 = getLuminance(rgb1[0], rgb1[1], rgb1[2]);
  var lum2 = getLuminance(rgb2[0], rgb2[1], rgb2[2]);
  var brightest = Math.max(lum1, lum2);
  var darkest = Math.min(lum1, lum2);
  return ((brightest + 0.05) / (darkest + 0.05)).toFixed(2);
}

function submitted(event) {
  let formValid = true;
  const ids = ['clientId', 'clientSecret', 'environment', 'resultsEmail', 'redirectUrl'];
  ids.forEach(id => {
    let element = document.getElementById(id);
    let elementError = document.getElementById(`${id}Error`);
    // Required check
    if (element.value === '') {
      formValid = false;
      elementError.hidden = false;
    } else if (id === 'resultsEmail' && !emailRegex.test(String(element.value).toLowerCase())) {
      formValid = false;
      elementError.hidden = false;
    } else {
      elementError.hidden = true;
    }
  });

  // If CRM selected then "Your API key" should not be empty
  let crmSystemElement = document.getElementById('crmSystem');
  crmSettingIDs = [];
  if (crmSystemElement.value === 'S365') {
    crmSettingIDs = ['apiKey', 'organisationId'];
  } else if (crmSystemElement.value === 'OMS') {
    crmSettingIDs = ['APIUser', 'base64Secret', 'aud', 'brokerEmail', 'workflowName'];
  }
  crmSettingIDs.forEach(id => {
    let element = document.getElementById(id);
    let errorElement = document.getElementById(`${id}Error`);
    if (element.value === '') {
      formValid = false;
      errorElement.hidden = false;
    } else {
      errorElement.hidden = true;
    }
  });

  const contrast = getContrast(document.getElementById('color').value);
  if (contrast < 4.5) {
    formValid = false;
  }

  if (!formValid) {
    event.preventDefault();
  }
}

function addClass(el) {
  el.classList.add('changed');
}

function crmChanged() {
  const crmSystemElement = document.getElementById('crmSystem');
  const noCRMSelected = document.getElementById('no-crm-selected');

  const S365Settings = document.getElementById('S365_settings');
  const OMSSettings = document.getElementById('OMS_settings');

  if (crmSystemElement.value === '') {
    noCRMSelected.classList.remove('d-none');
    S365Settings.classList.add('d-none');
    OMSSettings.classList.add('d-none');
  } else if (crmSystemElement.value === 'S365') {
    noCRMSelected.classList.add('d-none');
    S365Settings.classList.remove('d-none');
    OMSSettings.classList.add('d-none');
  } else if (crmSystemElement.value === 'OMS') {
    noCRMSelected.classList.add('d-none');
    S365Settings.classList.add('d-none');
    OMSSettings.classList.remove('d-none');
  }
}

function checkEnableTestButton() {
  const clientId = document.getElementById('clientId').value.trim();
  const clientSecret = document.getElementById('clientSecret').value.trim();
  const environment = document.getElementById('environment').value.trim();

  const testCredentialsBtn = document.getElementById('testCredentialsBtn');
  if (clientId && clientSecret && environment) {
    testCredentialsBtn.disabled = false;
  } else {
    testCredentialsBtn.disabled = true;
  }
}

function testCredentials() {
  const credentialsLoad = document.getElementById('credentials_load');
  const credentialsPass = document.getElementById('credentials_pass');
  const credentialsFail = document.getElementById('credentials_fail');
  credentialsPass.classList.add('d-none');
  credentialsFail.classList.add('d-none');

  const clientId = document.getElementById('clientId').value;
  const clientSecret = document.getElementById('clientSecret').value
  const environment = document.getElementById('environment').value
  credentialsLoad.classList.remove('d-none');

  setTimeout(function(){
    var xhr = new XMLHttpRequest();
    xhr.open('POST', '<?php echo get_rest_url() ?>mc_api/v1/test_credentials', false); // false for synchronous request
    xhr.setRequestHeader('Content-Type', 'application/json');
    xhr.send(JSON.stringify({
      environment: environment,
      clientId: clientId,
      clientSecret: clientSecret
    }));
    credentialsLoad.classList.add('d-none');
    if (xhr.responseText.trim() === 'true') {
      credentialsPass.classList.remove('d-none');
      return;
    }
    credentialsFail.classList.remove('d-none');
  }, 1000);
}

function checkEnableCrmTestButton() {
  const crmSystem = document.getElementById('crmSystem').value;
  let pass = false;
  if (crmSystem === 'S365') {
    pass =
      document.getElementById('apiKey').value.trim() &&
      document.getElementById('organisationId').value.trim();
  } else if (crmSystem === 'OMS') {
    pass =
      document.getElementById('APIUser').value.trim() &&
      document.getElementById('base64Secret').value.trim() &&
      document.getElementById('aud').value.trim() &&
      document.getElementById('brokerEmail').value.trim() &&
      document.getElementById('workflowName').value.trim();
  }

  const testCrmBtn = document.getElementById('testCrmBtn');
  testCrmBtn.disabled = !pass;
}

function testCrm() {
  const crmLoad = document.getElementById('crm_load');
  const crmPass = document.getElementById('crm_pass');
  const crmFail = document.getElementById('crm_fail');
  crmPass.classList.add('d-none');
  crmFail.classList.add('d-none');

  const crmSystem = document.getElementById('crmSystem').value;
  let post = {};
  if (crmSystem === 'S365') {
    post = {
      apiKey: document.getElementById('apiKey').value,
      organisationId: document.getElementById('organisationId').value
    }
  } else if (crmSystem === 'OMS') {
    post = {
      APIUser: document.getElementById('APIUser').value,
      base64Secret: document.getElementById('base64Secret').value,
      aud: document.getElementById('aud').value,
      brokerEmail: document.getElementById('brokerEmail').value,
      workflowName: document.getElementById('workflowName').value
    }
  }
  crmLoad.classList.remove('d-none');

  setTimeout(function(){
    var xhr = new XMLHttpRequest();
    xhr.open('POST', '<?php echo get_rest_url() ?>mc_api/v1/test_crm_credentials', false); // false for synchronous request
    xhr.setRequestHeader('Content-Type', 'application/json');
    xhr.send(JSON.stringify(post));
    crmLoad.classList.add('d-none');
    if (xhr.responseText.trim() === 'true') {
      crmPass.classList.remove('d-none');
      return;
    }
    crmFail.classList.remove('d-none');
  }, 1000);
}
